#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/stat.h>
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>

#define WIDTH 960 
#define HEIGHT 540
#define BPP 2
#define DEPTH 16

void setpixel(SDL_Surface *screen, int x, int y, Uint8 r, Uint8 g, Uint8 b)
{
	Uint32 *pixmem32;
	Uint32 colour;  

	colour = SDL_MapRGB( screen->format, r, g, b );

	pixmem32 = (Uint32*) screen->pixels  + y + x;
	*pixmem32 = colour;

}


void DrawScreen(SDL_Surface* screen, int h)
{ 
	int x, y, ytimesw;

	if(SDL_MUSTLOCK(screen)) 
	{
		if(SDL_LockSurface(screen) < 0) return;
	}

	for(y = 0; y < screen->h; y++ ) 
	{
		ytimesw = y*screen->pitch/BPP;
		for( x = 0; x < screen->w; x++ ) 
		{
			setpixel(screen, x, ytimesw, (x*x)/256+3*y+h, (y*y)/256+x+h, h);
		}
	}

	if(SDL_MUSTLOCK(screen)) SDL_UnlockSurface(screen);

	SDL_Flip(screen); 
}

int Game_Main(const char *path, const char *udn)
{
  int i;
	SDL_Surface *screen;
  SDL_Surface *btsu;
  char *fpath="/mtd_appdata/Font/shadow.ttf";
  struct stat sts;
  TTF_Font *font;
  SDL_Color c_black={ 0, 0, 0 };
  SDL_Color c_yellow={ 255, 255, 0 };
  SDL_Color c_white={ 255, 255, 255 };
  
  printf("**Sdltest Game_Main\n");
  if (stat(fpath, &sts)) {
    printf("TTF font not found\n");
    return(-1);
  }

	if (SDL_Init(SDL_INIT_VIDEO|SDL_INIT_TIMER) < 0 ) {
    printf("SDL_Init failed\n");
		SDL_Quit();  // without Quit TV cannot restart launcher app
                // Quit throws segmentation fault, but launcher can continue
    return 1;
  }
	screen= SDL_SetVideoMode(WIDTH, HEIGHT, DEPTH, SDL_HWSURFACE);
  if (! screen) {  
    printf("SDL_SetVideoMode failed\n");
		SDL_Quit();
		return 1;
	}

  if (TTF_Init()) {
    printf("TTF_Init failed\n");
		SDL_Quit();
		return 1;
	}

  font= TTF_OpenFont(fpath, 24);
  if (! font) {
    printf("TTF_OpenFont failed\n");
		SDL_Quit();
		return 1;
	}
  
  btsu= TTF_RenderUTF8_Shaded(font, "SDL text test", c_white, c_black);
  {
    SDL_Rect dstrect = { 20, 20, 0, 0 };
    SDL_BlitSurface(btsu, NULL, screen, &dstrect);
  }
  SDL_FreeSurface(btsu);

  btsu= TTF_RenderUTF8_Shaded(font, "row 500", c_black, c_yellow);
  {
    SDL_Rect dstrect = { 20, 500, 0, 0 };
    SDL_BlitSurface(btsu, NULL, screen, &dstrect);
  }
  SDL_FreeSurface(btsu);

  btsu= TTF_RenderUTF8_Shaded(font, "col 800", c_black, c_white);
  {
    SDL_Rect dstrect = { 800, 20, 0, 0 };
    SDL_BlitSurface(btsu, NULL, screen, &dstrect);
  }
  SDL_FreeSurface(btsu);

  SDL_Rect dstrect = { WIDTH/2-20, HEIGHT/2-10, 40, 20 };
	SDL_FillRect(screen, &dstrect, SDL_MapRGB(screen->format,255,100,100));
  SDL_Flip(screen);
  SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);
  int t= 9999;
  while (SDL_GetTicks() < t) {
    SDL_Event ev;
    if (SDL_PollEvent(&ev)) {
      printf("event %d\n", ev.type);
      if (ev.type == SDL_KEYDOWN) {
        SDL_Rect oldrect= dstrect;
        switch (ev.key.keysym.sym) {
        case SDLK_LEFT:
          dstrect.x-=5;
          break;
        case SDLK_RIGHT:
          dstrect.x+=5;
          break;
        case SDLK_UP:
          dstrect.y--;
          break;
        case SDLK_DOWN:
          dstrect.y++;
          break;
        default:
          break;                  
        }
      	SDL_FillRect(screen, &oldrect, SDL_MapRGB(screen->format,0,0,0));
      	SDL_FillRect(screen, &dstrect, SDL_MapRGB(screen->format,255,100,100));
        SDL_Flip(screen);
        if (ev.key.keysym.sym == SDLK_ESCAPE) t= 0; else t= SDL_GetTicks() + 9999;
      }
    }    
  }

  for(i=0;i<50;i++) {
    DrawScreen(screen,i++);
  }
    
  TTF_CloseFont(font);

	SDL_Quit();

	return 0;
}

void __attribute__ ((constructor)) test_init(void);
void test_init(void)
{
  printf("\n******** add-symbol-file fix/libSdltest.so %p\n", &setpixel-0xc54+0x8a0);
}
